# -*- coding: utf-8 -*-
'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

import os
import pygame

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))
image_dir = os.path.join(data_dir,  'images')

def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)

def load_image(filename, colorkey = None,  where="images"):
    if where == "images":
        fullpath = os.path.join(image_dir,  filename)
    
    try:
        image = pygame.image.load(fullpath)
    except pygame.error:
        raise
        
    image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0, 0))
        image.set_colorkey(colorkey,  pygame.RLEACCEL)
    return image
        
        
