# -*- coding: utf-8 -*-
import pygame

import data
import env

def lookup(alias):
    """Get the base name for an alias."""
    if alias in objs:
        return alias # It is already the correct name
    elif alias in aliases:
        return aliases[alias]
    else:
        return None

class Object():
    def __init__(self):
        self.name = ""
        self.takable = True # You may pick this item up
            
    def location_description(self): # Description to print along with the area's description
        return "There is a " + self.name + " here."

    def fullname(self):
        """Return the full name of the objects as in "a broken cat that you found\""""
        return self.name

    def description(self):
        return ""
    
    def use(self, object_id=None):
        """If object is defined, we use that object on this one."""
        
        if object_id is None:
            return "%s cannot be used." % self.name.capitalize()
        else:
            obj = lookup(object_id)
            return "%s cannot be used on %s." % (obj.capitalize(), self.name)

class cardBoardBox(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "cardboard box" 
        
    def location_description(self):
        return "There is a cardboard box here."
        
    def description(self):
        return "It is a big, brown box made of cardboard."

class food(Object):
    def __init__(self):
        Object.__init__(self)
        self.name = "your dinner"
        
    def location_description(self):
        return "There is a plate of food here."
        
    def description(self):
        return "It looks disgusting."

        
class bajs(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "bajs" 
        
    def location_description(self):
        return "There is a pile of bajs here."
        
    def description(self):
        return "It looks like a pile of Bajs.\nQuite disgusting"

class redString(Object):
    def __init__(self):
        Object.__init__(self)
        
        self.takable = False
        self.name = "string" 
        
    def location_description(self):
        return ""
        
    def description(self):
        return "It's a red string. It continues out of sight."

class ladder(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "broken ladder"
        self.parts = { "pipe" : False,
                       "table leg" : False,
                       "tree branch" : False,
                       "baguette" : False,
                       }

        self.complete = False

    def description(self):
        if not self.complete:
            s = "A broken ladder. It lacks some of its rungs."
        else:
            s = "A ladder mended with a steel pipe, a table leg, a branch and a baguette. Use it before you eat it."

        return s

    def use(self, obj=None):

        obj_id = lookup(obj)

        if obj_id is not None:
            if not self.complete:
                if obj_id == "pipe":
                    self.parts["pipe"] = True
                    if obj_id in env.env["current_area"].objects:
                        del env.env["current_area"].objects[obj_id]
                    elif obj_id in env.env["inventory"]:
                        del env.env["inventory"][obj_id]
                    ret = "You stick the pipe in one of the holes. It fits perfectly."
                elif obj_id == "table leg":
                    self.parts["table leg"] = True
                    if obj_id in env.env["current_area"].objects:
                        del env.env["current_area"].objects[obj_id]
                    elif obj_id in env.env["inventory"]:
                        del env.env["inventory"][obj_id]
                    ret = "You stick the table leg into one of the holes. It fits perfectly."
                elif obj_id == "tree branch":
                    self.parts["tree branch"] = True
                    if obj_id in env.env["current_area"].objects:
                        del env.env["current_area"].objects[obj_id]
                    elif obj_id in env.env["inventory"]:
                        del env.env["inventory"][obj_id]
                    ret = "You stick the branch into one of the holes. It fits perfectly."
                elif obj_id == "baguette":
                    self.parts["baguette"] = True
                    if obj_id in env.env["current_area"].objects:
                        del env.env["current_area"].objects[obj_id]
                    elif obj_id in env.env["inventory"]:
                        del env.env["inventory"][obj_id]
                    ret = "You stick the baguette into one of the holes. It kind of fits. You eat the crumbs that fall off."

                else:
                    ret = "You stick the object into one of the holes. It doesn't fit at all."

                self.complete = True
                for part in self.parts.itervalues():
                    self.complete &= part
                if self.complete:
                    self.name = "mended ladder"

                return ret

            else:
                return "There are no holes left."

        else:
            if self.complete:
                area = env.env["current_area"]
                if area.name == "low_wall":
                    env.env["accomplishments"]["used_ladder"] = True
                    if "ladder" in env.env["inventory"]:
                        del env.env["inventory"]["ladder"]
                    elif "ladder" in env.env["current_area"].objects:
                        del env.env["current_area"].objects["ladder"]
                    return "You put the ladder to the wall."
                elif area.name in ["high_wall", "high_wall2", "high_wall3"]:
                    return "The ladder is too short!"
                else:
                    return "There is no use for that here."
            else:
                return "The ladder is broken."

class pipe(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "steel pipe"
        
    def description(self):
        return "It's a steel pipe."

class baguette(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "baguette"
        
    def description(self):
        return "Looks delicious..."
    
    def use(self, obj_id=None):
        return "You need a drink to go with it!"

class tableLeg(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "table leg"
        
    def description(self):
        return "A leg from a broken wooden table."

class treeBranch(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "tree branch"
        
    def description(self):
        return "A thin but strong branch from an oak tree."

class mustardGas(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "mustard gas"

    def location_description(self):
        return "There is a canister of mustard gas here."
        
    def description(self):
        return "The label says GVP Inc. novelty sulfur mustard."

class gateOpener(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "gateopener"
        
    def location_description(self):
        return "There is a gateopener here."
        
    def description(self):
        return "An intricate tool, apparently used for opening a gate. No, you can't just walk around it!"

class manhole(Object):
    def __init__(self):
        Object.__init__(self)
        self.name = "manhole"
        self.takable = False
        
    def description(self):
        "A hatch is concealing an opening in the street."

class flashlight(Object):
    def __init__(self):
        Object.__init__(self)
        self.name = "flashlight"
        self.on = False
      
    def description(self):
        text = "It's a regular flashlight."
        if not self.on: 
            text += " The lamp is turned of."
        else:
            text += " The lamp is turned on."
        return text
    
    def use(self, obj=None):
        self.on = True
        return "You turn the flashlight on."
        
        
class compass(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "Compass"
        
    def description(self):
        return "It looks old. Maybe it won't work as well as you were hoping."

class stoneTablets(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "tablets"
        self.takable = False
    def location_description(self):
        return ""

    
    def description(self):
        return """The five stone tablets read:
How long is the string?
Where does it end?
Why are you following the string?
What would have happened if hadn't?
Who created the string?"""

class scissors(Object):
    def __init__(self):
        Object.__init__(self)

        self.name = "pair of scissors"
    
    def description(self):
        return "It appears to be a regular pair of scissors"

aliases = { "poo" : "bajs",
            "shit" : "bajs",
            
            "red string":"string", 
            "the fucking string" : "string", 
            
            "box" : "cardboard box",
            "cardboard" : "cardboard box",
            
            "dinner" : "food",
            "plate" : "food",

            "string" : "string", # Actually, there is no string object, because it exists in multiple areas and behaves differently, this is a small hack, albeit a nice one
            
            "broken ladder" : "ladder",
            "branch" : "tree branch",
            "steel pipe" : "pipe", 
            "steelpipe" : "pipe",
            "leg" : "table leg",
            
            "gas" : "mustard gas",
            "canister" : "mustard gas",
            
            "light" : "flashlight",
            "torch" : "flashlight",

            "stone tablets" : "tablets", 
            "the five stone tablets" : "tablets", 
            "scissors" :  "pair of scissors", 
           }

filenames = { "bajs" : "bajs.jpg", # I admit that this part is a little ugly, and that it cannot be easily scripted. It is easy to change, though, given some time.       
              "cardboard box" : "bajs.jpg",
             }

# Only add the name of the class here. Do not create an instance
objs = {"bajs": bajs,
        "string" : redString,
        "cardboard box" : cardBoardBox,
        "food" : food,
        "ladder" : ladder,
        "baguette" : baguette,
        "tree branch" : treeBranch,
        "table leg" : tableLeg,
        "pipe" : pipe,
        "mustard gas" : mustardGas,
        "gateopener" : gateOpener,
        "manhole" : manhole, 
        "flashlight" : flashlight, 
        "compass" : compass,
        "tablets" : stoneTablets,
        "pair of scissors" : scissors, 
        }
