# -*- coding: utf-8 -*-
# utils.py

import pygame
from pygame.locals import *
import cPickle
import os

import env # For saving
import data

pygame.font.init()

class Util:
    fonts = {"uglyquaitalic16FalseFalse" : pygame.font.Font(os.path.join(data.data_dir, "uglyquaitalic.ttf"), 16),
             "uglyqua16FalseFalse" : pygame.font.Font(os.path.join(data.data_dir, "uglyqua.ttf"), 16)}


def render_text(text, maxwidth, color=(0, 0, 0), bg_color=None, size=16, font_name="Arial", bold=False, italic=False):
    """
    Returns a surface with the rendered text.
    
    This function could and should be expanded a lot. 
    """
    
    surf = render_lines(text, maxwidth, color, bg_color, size, font_name, bold, italic)
    return surf

def wrap_string(s, font, maxwidth):

    lines = s.split("\n") # We can add other newlines too, naturally
    retlines = []
    for line in lines:
        words = line.split(" ")
        start = 0

        i = 0 
        while i < len(words): # Changed to a while loop, because we have to roll back the index
            width = font.size(" ".join(words[start:i+1]))[0] # Check if next word makes it too wide
            if width > maxwidth:  # Insert a newline if the line is too wide
                retlines.append(" ".join(words[start:i])) 
                start = i  # Begin again on the next word
                i -= 1
            elif i is len(words) - 1: # Newline if we reached the last word of the line without it running too wide
                retlines.append(" ".join(words[start:i + 1]))  

            i += 1

    if retlines:
        return retlines
    else:
        return [""]

def render_lines(text, maxwidth, color, bg_color, size, font_name, bold, italic):
    """Lines is a list."""

    # Okay, fuck it. The perpixelalpha will not work, for some reason. I sneeze on that and force you to supply a bgcolor.
    # If you put back the caching, be aware that this will not account for the backgoround color

    # We cache fonts
    font_id = font_name + str(size) + str(bold) + str(italic)
    font = Util.fonts[font_id]

    lines = wrap_string(text, font, maxwidth)

    width = max(font.size(line)[0] for line in lines)
    height = sum(font.size(line)[1] for line in lines)
                            
    message_surf = pygame.Surface((width, height))
    message_surf.fill(bg_color)

    last_height = 0
    for line in lines:
        line_surf = Util.fonts[font_id].render(line, True, color, bg_color)
        message_surf.blit(line_surf, (0, last_height))
        last_height += line_surf.get_height()

    message_surf.convert()
    # Due to a bug that I have no idea how to resolve now, per pixel transparencies do not work
    # when blitting to message_surf. This is why I have turned of antialiasing.
    return message_surf

def get_width(text,  font, size=16, bold=False, italic=False): 
    font_id = font +str(size) + str(bold) + str(italic)
    try:
        dafont = Util.fonts[font_id]
        return dafont.size(text)[0]
    except IndexError:
        return 0

    
    
def add_message(text):
    """Small hack to allow areas (and other functions) to print to the game loop descriptionbox."""
    e = pygame.event.Event(USEREVENT, {"message_type": "output", "message": text})
    pygame.event.post(e)

def save_game(filename):
    """Save game state to file."""
    try:
        f = file(filename, "wb")
        cPickle.dump(env.env, f)
        f.close()
        return "Game saved to %s" % filename
    except IOError:
        return "Could not save file %s because an IO error occurred." % filename

def load_game(filename):
    """Save game state to file."""
    try:
        f = file(filename, "rb")
        tempenv = cPickle.load(f)
        f.close()

        if not isinstance(tempenv, dict) or \
                not "current_area" in tempenv or \
                not "inventory" in tempenv or \
                not "areas" in tempenv or \
                not "accomplishments" in tempenv:
            raise TypeError("Invalid file format")

        # File is correct and exists
        env.env = tempenv

        # We also restore the music, because there are areas that do not set their
        import sound
        music = env.env["nowplaying"]
        sound.stop_music() # Stop the old music. This makes env forget what we were playing
        sound.play_music(music) # Restore state of background music

        env.env["current_area"].update() # We need to do this to update the cache with our current images

        return 0, "Game loaded"
    except IOError, err:
        return 1, "There is no such file"
    except cPickle.BadPickleGet:
        return 1, "Invalid file format"
    except TypeError:
        return 1, "Invalid file format"
